import os
from functools import wraps
from fastapi import Request, Body

from magicapi.Models.Task import Task, TaskParams

import inspect
from fastapi import HTTPException, APIRouter
import json
from datetime import datetime

from magicapi.Utils.random_utils import random_str
from magicapi.Errors import BackendException

from magicapi import g

from magicapi.Decorators.helpers import async_safe


def run_in_background(f):
    router_path = f"/run_in_background/{f.__name__}"
    new_r = APIRouter()

    @new_r.post(router_path, tags=["background_tasks"], include_in_schema=False)
    async def endpoint(
            request: Request,
            task_id: str = Body(...),
            secret_token: str = Body(...),
            params: str = Body(...),
    ):
        if g.settings.print_level > 1:
            print("endpoint just received!", "params", params)
        # get the task from dynamo
        response = Task.get_table().get_item(Key={"task_id": task_id})
        task_dict = response.get("Item")
        if not task_dict:
            raise HTTPException(status_code=404, detail="Invalid task id.")

        task = Task(**task_dict)
        if not task or secret_token != task.secret_token:
            raise HTTPException(status_code=404, detail="Invalid task request.")

        if task.status != "in-lambda":
            # when local sometimes this happens before lambda changes queued to in-lambda
            if not task.local:
                raise HTTPException(
                    status_code=404, detail="This task was already completed."
                )

        j_params = json.loads(params)
        args = j_params.get("args", [])
        kwargs = j_params.get("kwargs", {})
        if g.settings.print_level > 1:
            print("inspect", inspect.signature(f), "a", args, "k", kwargs)

        await async_safe(f, *args, **kwargs)

        return {"success": True, "message": ""}

    g.app.include_router(new_r)

    @wraps(f)
    def wrapper(*args, **kwargs):
        if not os.getenv("HasDB"):
            raise BackendException(
                message="You cannot add tasks if you do not add a DB!"
            )

        if g.settings.print_level > 1:
            print("given to function", "args", args, "kwargs", kwargs)
        task_params = TaskParams(args=list(args), kwargs=kwargs)
        now = datetime.utcnow()
        task = Task(
            task_id=random_str(30),
            url=g.base_url + router_path,
            status="queued",
            sent=False,
            secret_token=random_str(50),
            created_at=now,
            last_updated=now,
            params=task_params.json(),
            local=g.settings.local,
        )
        g.tasks.append(task)

        return task.task_id

    return wrapper



from magicdb.Models import MagicModel
from pydantic import EmailStr


class CurrentUser(MagicModel):
    uid: str
    phone_number: str = None
    email: EmailStr = None
    email_verified: bool = None
    auth_time: int
    iat: int
    exp: int
    iss: str = None
    aud: str = None
    user_id: str = None
    sub: str = None
    firebase: dict = {}



from twilio.rest import Client
import os
from functools import wraps
# from magicapi import settings
from magicapi import g

from magicapi.Errors import TwilioException


try:
    client = Client(g.settings.twilio_account_sid, g.settings.twilio_auth_token)
except Exception as e:
    print("Twilio SID or AUTH TOKEN not given.")

FROM_NUMBER = os.environ.get("FROM_NUMBER")


def need_twilio_vars(f):
    @wraps(f)
    def wrapper(*args, **kwargs):
        twilio_vars = [g.settings.twilio_account_sid, g.settings.twilio_auth_token]
        if None in twilio_vars:
            raise TwilioException(
                message="Not all Twilio credentials found in .env file. Need TWILIO_ACCOUNT_SID and TWILIO_AUTH_TOKEN."
            )
        return f(*args, **kwargs)

    return wrapper


def need_copilot_vars(f):
    @wraps(f)
    def wrapper(*args, **kwargs):
        twilio_vars = [g.settings.twilio_messaging_service_sid]
        if None in twilio_vars:
            raise TwilioException(
                message="Not all Twilio co-pilot credentials found in .env file. Need MESSAGING_SERVICE_SID."
            )
        return f(*args, **kwargs)

    return wrapper


@need_twilio_vars
def send_text(to, body, from_number=FROM_NUMBER):
    message = client.messages.create(body=body, from_=from_number, to=to,)
    return message.sid


@need_twilio_vars
def send_text_with_copilot(to, body, status_callback=g.settings.twilio_status_callback):
    message = client.messages.create(
        body=body,
        messaging_service_sid=g.settings.twilio_messaging_service_sid,
        to=to,
        status_callback=status_callback,
    )
    return message.sid



# from magicapi import settings
from magicapi import g

email_body = f"""Hello,

We received a request to sign in to {g.settings.company_name} using this email address. If you want to sign in with <EMAIL_ADDRESS>, click this link:

<LINK>.

If you did not request this link, you can safely ignore this email.

Thanks,
The {g.settings.company_name} Team
"""

subject = f"Sign in to {g.settings.company_name}"


def make_template_and_subject(email_address: str, magic_link: str):
    new_body = email_body.replace("<LINK>", magic_link).replace(
        "<EMAIL_ADDRESS>", email_address
    )
    return new_body, subject



from fastapi import Request

from magicapi import g

from fastapi.responses import JSONResponse


class MagicException(Exception):
    def __init__(self, message: str):
        self.status_code: int = 452
        self.message: str = message


class BackendException(MagicException):
    pass


class FrontendException(MagicException):
    pass


class FirestoreException(MagicException):
    pass


class TwilioException(MagicException):
    pass


@g.app.exception_handler(MagicException)
def backend_exception_handler(request: Request, exc: MagicException):
    return JSONResponse(
        status_code=exc.status_code,
        content={"success": False, "message": f"{exc.message}"},
    )



__version__ = "0.1.0"

"""Imports the Routes and the app and glues them all together"""
# from .config import settings
from magicapi.Globals.G import g

from magicapi.app_factory import create_app, create_handler, add_magic_routers
